Module fulltext
/***************************************************************************
                        fulltext.mod  -  description
                             -------------------
    begin                : 2001
    copyright            : (C) 2004 by tdb Software Service
    email                : tf@tdb.de

		$Id: fulltext.mod,v 1.1 2005/09/08 07:34:11 path Exp $
 ***************************************************************************

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*<doku>
<author>Horst Klier, Hermann Gtz</author>
<created>11.10.2001</created>
<changed>TF / 2004-07-15 [cleanup / sysbase reachable from root dir, too]</changed>
Rev. 12
Hinweis: Benutzt werden die workdef-Dateien. Diese werden vom Baseman angelegt und gepflegt.
In der DEF:
<code>
[FULLTEXT]
fulltext=JA
..oder
FULLTEXT=JA
</code>
Mssen nicht angegeben werden:
<code>
Fields=Name
ExtABC=0123456789
Cut=500
Kontraindex=
Step=0
Modus=4
MaskenFeld=-1
DynKontraIndex=
</code>
Die Parameter entsprechen der tdb-engine-Volltext-Doku
</doku>*/

procedure Fulltext_Test(cName:STRING):REAL
/*</doku>
Ermittelt ob Parameter fr Volltext in der DEF-Datei vorliegen
Beispiel:
<pre>
if TestFulltext('firma')=JA
   Irgendwas
end
</pre>
<return>
JA   - Volltextinformationen vorhanden
NEIN - Keine Volltextinformationen vorhanden
</return>
<changed>TF / 2004-07-15</changed>
</doku>*/
  VAR nRet : REAL
  VAR cDef : STRING
  if IsFile('workdef/'+cName+'.def')
    cDef:='workdef/'+cName+'.def'
  elsif IsFile('../workdef/'+cName+'.def')
    cDef:='../workdef/'+cName+'.def'
  elsif IsFile('../workdef/system/'+cName+'.def')
    cDef:='../workdef/system/'+cName+'.def'
  elsif IsFile('workdef/system/'+cName+'.def')
    cDef:='workdef/system/'+cName+'.def'
  else
    cDef:=''
  end
  //if cDef='' or GetIdent(cDef,'FULLTEXT.fulltext')=''
  //if cDef='' or (~GetIdent(cDef,'FULLTEXT.fulltext'),~GetIdent(cDef,'FULLTEXT.FULLTEXT'))
  If cDef='' or ~(GetIdent(cDef,'FULLTEXT.fulltext') or GetIdent(cDef,'FULLTEXT.FULLTEXT'))
    nRet:=NEIN
  else
    nRet:=JA
  end
  Return nRet
endproc


procedure Fulltext_GetParam(cName:STRING;VAR cFields,cExtABC:STRING;VAR nCut:REAL;VAR cKontraindex:STRING;VAR nModus,nStep,nMaskenfeld:REAL;VAR cDynKontraIndex:STRING):REAL
/*</doku>
Ermittelt Parameter fr Volltext aus der DEF-Datei
Aufruf:
<code>
VAR cFields,cExtABC,cKontraindex,cDynKontraIndex : STRING
VAR nCut,nModus,nStep,nMaskenfeld : REAL
GetFulltextParam(cName,cFields,cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)
</code>
Beispiel:
<pre>
VAR cFields,cExtABC,cKontraindex,cDynKontraIndex : STRING
VAR nCut,nModus,nStep,nMaskenfeld : REAL
if GetFulltextParam('firma',cFields,cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)=JA
  nErr:=ScanRecs(dbTAB,dbSUCHWORT,dbSUCHREL,Fields(cFields),cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)
end
</pre>
<return>
 JA   - Volltextinformationen vorhanden
 NEIN - Keine Volltextinformationen vorhanden
</return>
</doku>*/
  VAR nRet : REAL
  VAR cDef : STRING

  nRet:=Fulltext_Test(cName)
  if nRet=JA

    if IsFile('workdef/'+cName+'.def')
      cDef:='workdef/'+cName+'.def'
    elsif IsFile('../workdef/'+cName+'.def')
      cDef:='../workdef/'+cName+'.def'
    elsif IsFile('../workdef/system/'+cName+'.def')
      cDef:='../workdef/system/'+cName+'.def'
    elsif IsFile('workdef/system/'+cName+'.def')
      cDef:='workdef/system/'+cName+'.def'
    end

    .EC 1
    cFields:=GetIdent(cDef,'FULLTEXT.Fields')
    cExtABC:=GetIdent(cDef,'FULLTEXT.ExtABC')
    nCut:=VAL(GetIdent(cDef,'FULLTEXT.Cut'))
    cKontraindex:=GetIdent(cDef,'FULLTEXT.Kontraindex')
    nModus:=VAL(GetIdent(cDef,'FULLTEXT.Modus'))
    nStep:=VAL(GetIdent(cDef,'FULLTEXT.Step'))
    nMaskenfeld:=VAL(GetIdent(cDef,'FULLTEXT.MaskenFeld'))
    cDynKontraIndex:=GetIdent(cDef,'FULLTEXT.DynKontraIndex')
    .EC 0

    if cFields=''
      cFields:='complete'
    end
    if cExtABC=''
      cExtABC:='0123456789.'
    end
    if nCut=0
      nCut:=1000000
    end
    if cKontraindex=''
      cKontraindex:=''
    end
    if nModus=0
      nModus:=4
    end
    if nStep=0
      nStep:=0
    end
    if nMaskenfeld=0
      nMaskenfeld:=0
    end
    if cDynKontraIndex=''
      cDynKontraIndex:=''
    end
  end

  Return nRet
endproc

procedure FulltextSub_OpenFTBases(cName:STRING;VAR dbSUCHWORT,dbSUCHREL:REAL):REAL
/*<doku>
Subroutine zum ffnen der Volltexttabellen
Wird nur Intern verwenden, nicht direkt aufzurufen
<return>
0 oder Fehlernummer beim ffnen
Rckgabe der Handles in den bergebenen Parametern
</return>
<changed>TF / 2004-07-15</changed>
</doku>*/
  VAR cVTDir : STRING
  VAR nErr : REAL
  if IsFile('database/'+cName+'.dat')
    cVTDir:='database/'+cName+'_vt'
  elsif IsFile('../database/'+cName+'.dat')
    cVTDir:='../database/'+cName+'_vt'
  elsif IsFile('sysbase/'+cName+'.dat')
    cVTDir:='sysbase/'+cName+'_vt'
  else
    cVTDir:='../sysbase/'+cName+'_vt'
  end

  dbSUCHWORT:=OpenDB(cVTDir+'/d-ind.dat',"",0,15)
  if dbSUCHWORT#0
    dbSUCHREL:=OpenDB(cVTDir+'/d-rel.rel',"",0,15)
    if dbSUCHREL#0
      nErr:=0
    else
      nErr:=$Fehler
      dbSUCHREL:=0
    end
  else
    nErr:=$Fehler
    dbSUCHWORT:=0
  end
  Return nErr
endproc

procedure FulltextSub_CloseFTBases(VAR dbSUCHWORT,dbSUCHREL:REAL)
/*<doku>
   Subroutine zum Schliessen der Volltexttabellen
   Wird nur Intern verwenden, nicht direkt aufzurufen
</doku>*/
  if dbSUCHREL#0
    CloseDB(dbSUCHREL)
    dbSUCHREL:=0
  end
  if dbSUCHWORT#0
    CloseDB(dbSUCHWORT)
    dbSUCHWORT:=0
  end
endproc

procedure Fulltext_ScanRecs(dbTAB:REAL):REAL
/*<doku>
Erzeugt kompletten Volltextindex
Stichwort- und Relationstabelle werden automatisch geffnet und geschlossen
<parameters>Tabellenhandle</parameters>
<return>0 oder Fehlernummer; ist kein Volltextindex definiert ebenfalls 0</return>
</doku>*/

  VAR cFields,cExtABC,cKontraindex,cDynKontraIndex : STRING
  VAR nCut,nModus,nStep,nMaskenfeld : REAL
  VAR nErr,dbSUCHWORT,dbSUCHREL : REAL
  VAR cName : String
  Var nFile : Real
  nFile := FileNr
  cName:=Exchange(Lower(DBName(dbTAB)),'.dat','')
	if Fulltext_GetParam(cName,cFields,cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)=JA
    nErr:=FulltextSub_OpenFTBases(cName,dbSUCHWORT,dbSUCHREL)
    if nErr=0
      PrimFile(dbTAB)
      .EM 1
..cgiwriteln('<!-- ScanRecs Parameter:')
..cgiwriteln('cName,cFields,cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex')
..cgiwriteln(cName+','+cFields+','+cExtABC+','+Str(nCut)+','+cKontraindex+','+Str(nModus)+','+Str(nStep)+','+Str(nMaskenfeld)+','+cDynKontraIndex)
..cgiwriteln('-->')
      nErr:=ScanRecs(dbTAB,dbSUCHWORT,dbSUCHREL,Fields(cFields),cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)
      ..nErr:=ScanRecs(dbTAB,dbSUCHWORT,dbSUCHREL,Fields(complete),cExtABC)
      .EM 0
      if nErr<0
        nErr:=$Fehler
      else
        nErr:=0
      end
    end
    FulltextSub_CloseFTBases(dbSUCHWORT,dbSUCHREL)
  else
    nErr:=0
  end
  PrimTable(nFile)
  Return nErr
endproc

procedure Fulltext_ScanRec(dbTAB:REAL):REAL
/*<doku>
Erzeugt Volltextindex fr aktuellen Datensatz
Stichwort- und Relationstabelle werden automatisch geffnet und geschlossen
<parameters>Tabellenhandle</parameters>
<return>0 oder Fehlernummer; ist kein Volltextindex definiert ebenfalls 0</return>
</doku>*/
  VAR cFields,cExtABC,cKontraindex,cDynKontraIndex : STRING
  VAR nCut,nModus,nStep,nMaskenfeld : REAL
  VAR nErr,dbSUCHWORT,dbSUCHREL : REAL
  VAR cName : STRING
  Var nFile : Real
  nFile := FileNr
  cName:=Exchange(Lower(DBName(dbTAB)),'.dat','')
  if Fulltext_GetParam(cName,cFields,cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)=JA
    nErr:=FulltextSub_OpenFTBases(cName,dbSUCHWORT,dbSUCHREL)
    if nErr=0
      PrimFile(dbTAB)
      .em 1
      nErr:=ScanRec(dbTAB,dbSUCHWORT,dbSUCHREL,Fields(cFields),cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)
      ..nErr:=ScanRec(dbTAB,dbSUCHWORT,dbSUCHREL,Fields(complete),cExtABC)
      .em 0
    end
    FulltextSub_CloseFTBases(dbSUCHWORT,dbSUCHREL)
  else
    nErr:=0
  end
  PrimTable(nFile)
  Return nErr
endproc

procedure Fulltext_UnScanRec(dbTAB:REAL):REAL
/*<doku>
Lscht Volltextindex fr aktuellen Datensatz
Stichwort- und Relationstabelle werden automatisch geffnet und geschlossen
<parameters>Tabellenhandle</parameters>
<return>0 oder Fehlernummer; ist kein Volltextindex definiert ebenfalls 0</return>
</doku>*/
  VAR cFields,cExtABC,cKontraindex,cDynKontraIndex : STRING
  VAR nCut,nModus,nStep,nMaskenfeld : REAL
  VAR nErr,dbSUCHWORT,dbSUCHREL : REAL
  VAR cName : STRING
  Var nFile : Real
  nFile := FileNr
  cName:=Exchange(Lower(DBName(dbTAB)),'.dat','')
  if Fulltext_GetParam(cName,cFields,cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)=JA
    nErr:=FulltextSub_OpenFTBases(cName,dbSUCHWORT,dbSUCHREL)
    if nErr=0
      PrimFile(dbTAB)
      .em 1
      nErr:=UnScanRec(dbTAB,dbSUCHWORT,dbSUCHREL,Fields(cFields),cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)
      ..nErr:=UnScanRec(dbTAB,dbSUCHWORT,dbSUCHREL,Fields(complete),cExtABC)
      .em 0
      if nErr>0
        nErr:=0
      end
    end
    FulltextSub_CloseFTBases(dbSUCHWORT,dbSUCHREL)
  else
    nErr:=0
  end
  PrimTable(nFile)
  Return nErr
endproc

procedure Fulltext_DelRec(dbTAB,nRec:REAL):REAL
/*<doku>
Lscht Datensatz
Dabei wird ein evtl. Volltextindex beachtet
<return>ein evtl. Fehler bei Volltextaktionen</return>
</doku>*/
  VAR nErr : REAL
  Var nFile : Real
  nFile := FileNr
  ReadRec(dbTAB,nRec)
  nErr:=Fulltext_UnScanRec(dbTAB)
  if nErr=0
    if nRec<FileSize(dbTAB)
      ReadRec(dbTAB,FileSize(dbTAB))
      nErr:=Fulltext_UnScanRec(dbTAB)
      if nErr=0
        DelRec(dbTAB,nRec)
        ReadRec(dbTAB,nRec)
        nErr:=Fulltext_ScanRec(dbTAB)
      end
    else
      DelRec(dbTAB,nRec)
    end
  end
  PrimTable(nFile)
  Return nErr
endproc

procedure Fulltext_MarkTable(dbTAB:REAL;cSuch:STRING):REAL
/*<doku>
Sucht ber Volltext in Tabelle dbTAB nach cSuch
<return>Returnwert von tdb-engine-Funktion MarkTable</return>
</doku>*/
  VAR cFields,cExtABC,cKontraindex,cDynKontraIndex : STRING
  VAR nCut,nModus,nStep,nMaskenfeld : REAL
  VAR nErr,dbSUCHWORT,dbSUCHREL : REAL
  VAR cName : STRING
  Var nFile : Real
  nFile := FileNr
  cName:=Exchange(Lower(DBName(dbTAB)),'.dat','')
  if Fulltext_GetParam(cName,cFields,cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)=JA
    nErr:=FulltextSub_OpenFTBases(cName,dbSUCHWORT,dbSUCHREL)
    if nErr=0
      PrimFile(dbTAB)
      .em 1
      nErr:=MarkTable(dbTAB,dbSUCHWORT,cSuch,cExtABC,'',nMaskenfeld,dbSUCHREL)
      .em 0
    end
    FulltextSub_CloseFTBases(dbSUCHWORT,dbSUCHREL)
  else
    nErr:=0
  end
  PrimTable(nFile)
  Return nErr
endproc

procedure Fulltext_MarkBits(dbTAB:REAL;cSuch:STRING;VAR aTBits:REAL[]):REAL
/*<doku>
Sucht ber Volltext in Tabelle dbTAB nach cSuch
<return>Returnwert von tdb-engine-Funktion MarkTable</return>
</doku>*/
  VAR cFields,cExtABC,cKontraindex,cDynKontraIndex : STRING
  VAR nCut,nModus,nStep,nMaskenfeld : REAL
  VAR nErr,dbSUCHWORT,dbSUCHREL : REAL
  VAR cName : STRING
  Var nFile : Real
  nFile := FileNr
  cName:=Exchange(Lower(DBName(dbTAB)),'.dat','')
  if Fulltext_GetParam(cName,cFields,cExtABC,nCut,cKontraindex,nModus,nStep,nMaskenfeld,cDynKontraIndex)=JA
    nErr:=FulltextSub_OpenFTBases(cName,dbSUCHWORT,dbSUCHREL)
    if nErr=0
      PrimFile(dbTAB)
      .em 1
      nErr:=MarkBits(dbTAB,dbSUCHWORT,cSuch,cExtABC,'',nMaskenfeld,aTBits,dbSUCHREL)
      .em 0
    end
    FulltextSub_CloseFTBases(dbSUCHWORT,dbSUCHREL)
  else
    nErr:=0
  End
  PrimTable(nFile)
  Return nErr
endproc


procedure Fulltext_GetVersion:REAL
/*<doku>
<return>Revision</return>
</doku>*/
  Return 11
endproc

Procedure writeRecCurrentFT(db:Integer)
/*<doku>
<created date="27.02.2004 14:07" author="Sven Killig"/>
</doku>*/
/*dumpRecord(db,0)
CGIWriteLn('')*/
Fulltext_UnScanRec(db)
WriteRec(db,RecNo(db))
Fulltext_ScanRec(db)
EndProc