module macros

procedure macro_pagecount(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
	write(ft, Str(FileSize(dbWIKI)))
endproc


procedure macro_pagename(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
	var sWikiPage : STRING
	if sContext then
		sWikiPage := sContext +":"
	end
	sWikiPage := sWikiPage + sPageName
	write(ft, sWikiPage)
endproc


procedure macro_pagehits(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
	var sWikiPage : STRING
	if sContext then
		sWikiPage := sContext +":"
	end
	sWikiPage := sWikiPage + sPageName
	ReadRec(dbWIKI, FindRec(dbWIKI, sWikiPage, 1,1))
	write(ft, GetField(dbWIKI, "Hits"))
endproc


procedure macro_pagelog(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//last changelog entry of the currently displayed wiki page
	var sWikiPage : STRING
	if sContext then
		sWikiPage := sContext +":"
	end
	sWikiPage := sWikiPage + sPageName
		ReadRec(dbWIKI, FindRec(dbWIKI, sWikiPage, 1,1))
		write(ft, GetField(dbWIKI, "ChangeLog"))
endproc


procedure macro_pagestamp(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//last change stamp of the currently displayed wiki page
	var sWikiPage : STRING
	if sContext then
		sWikiPage := sContext +":"
	end
	sWikiPage := sWikiPage + sPageName
	ReadRec(dbWIKI, FindRec(dbWIKI, sWikiPage, 1,1))
	write(ft, formatStamp(GetRField(dbWIKI, "Stamp")))
endproc

procedure macro_pagesize(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//size in bytes of the currently displayed wiki page
	var sWikiPage : STRING
	if sContext then
		sWikiPage := sContext +":"
	end
	sWikiPage := sWikiPage + sPageName
	ReadRec(dbWIKI, FindRec(dbWIKI, sWikiPage, 1,1))
	write(ft, Str(MemoLen(dbWIKI, "Content")))
endproc

procedure macro_referrer(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//date time stamp of the GeboGebo installation
	write(ft, GetEnv("HTTP_REFERER"))
endproc

procedure macro_serverstamp(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//date time stamp of the GeboGebo installation's server
	write(ft, formatStamp(UNIX_NOW))
endproc

procedure macro_wikiname(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//name of the GeboGebo installation
		write(ft, sgWikiName)
endproc

procedure macro_wikiurl(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//base URL of the GeboGebo installation
	write(ft, sgGeboURL)
endproc

procedure macro_version(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//version of the GeboGebo installation
	write(ft, sgVersionStr)
endproc

procedure macro_usercount(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//number of registered users
	var dbUSER	: INTEGER

	dbUSER := OpenDB(sgDBDir+"user.dat")
	write(ft, Str(FileSize(dbUSER)))
	closedb(dbUSER)
endproc

procedure macro_subscribercount(dbWIKI,ft : INTEGER; sContext, sPageName : STRING)
//number of subscribers to the page
	var sWikiPage : STRING
	var iID				: INTEGER
	var dbSUBSCRIB: INTEGER

	if sContext then
		sWikiPage := sContext +":"
	end
	sWikiPage := sWikiPage + sPageName

	ReadRec(dbWIKI, FindRec(dbWIKI, sWikiPage, 1,1))
	iID := GetRField(dbWIKI, "ID")

	dbSUBSCRIB := OpenDB(sgDBDir+"subscrib.dat")
	if FindAndMark(dbSUBSCRIB, "$Wiki=0 OR $Wiki="+Str(iID)) > 0 then
		write(ft, Str(NMarks(dbSUBSCRIB)))
	else
		write(ft, "0")
	end
	closedb(dbSUBSCRIB)


endproc

procedure isMacroAllowed(sMacro : STRING) : INTEGER

		var bAllowed	: INTEGER = 0
		var fList			: INTEGER
		var s					: STRING

		if IsFile(sgMacrosList), GetSize(sgMacrosList)>0 then
			fList := Reset(sgMacrosList)
			repeat
				s:=lower(rtrim(readln(fList)))
				if s="*" OR s = sMacro or s="+"+sMacro then
					bAllowed := 1
				elsif s = "-"+sMacro then //definitely not allowed macro
					bAllowed := -1
				end
			until eot(fList) OR bAllowed # 0
			close(fList)
			if bAllowed < 0 then bAllowed := 0 end
		else //by default all macros are enabled as long as no macros.conf exists
			bAllowed := 1
		end
		return bAllowed
endproc

procedure substMacros(sGebo : STRING)
/*
Substitution of certain placeholders in the templates
They need to be like <''pagename''>. Just as if one was editing a wiki page, but enclosed by < and >.
*/
	var iBeg, iEnd			: INTEGER
	var sMacro, sOrigin : STRING
	var f								: INTEGER
	var dbWIKI 					: INTEGER
	var sParameter			: STRING
	var p,q							: INTEGER
	var bAllowed				: INTEGER


	dbWIKI := OpenDB(sgDBDir+"wiki.dat")
	iEnd := 1

	while iBeg := RAMTEXT_Find(sGebo,"<''",iEnd) do
		if iEnd := RAMTEXT_Find(sGebo,"''>",iBeg), iEnd - iBeg < 256 then
			RAMTEXT_Copy(sGebo, iBeg, iEnd - iBeg +3)
			sOrigin := RAMTEXT_Part("ramtext:~clip",1, GetSize("ramtext:~clip"))
			sMacro := RAMTEXT_Part("ramtext:~clip",4, GetSize("ramtext:~clip")-6)

			if p := Pos("(",sMacro)>0, q:=Pos(")",sMacro)>p then
				sParameter := sMacro[p+1,q-p-1]
				sMacro := sMacro[1,p-1]
			else
				sParameter:=""
			end

			sMacro := lower(sMacro)

			f := rewrite("ramtext:dynmacro")
			write(f,"<!-- macro: "+sMacro +" "+ sParameter +" //-->")

			if sMacro[1,7] = "plugin:" then // execute a macro-plugin
				sMacro := sMacro[8,255]
				Note(sParameter)
				TimeOut(igMacroTimeout)
				SetPara("ec 1")
				do _"ExecMacro("+sgPluginDir + sMacro+",macro("+Str(dbWIKI)+","+Str(f)+",'',''))"
				TimeOut(0)
				SetPara("ec 0")
				if TDB_LastError then
					write(f, "<span class='macrofault'>"+lang("badmacroplugin")+ " " +sMacro+"</span>")
				end
			else
				if bAllowed := isMacroAllowed(sMacro) then
					SetPara("ec 1")
					DO _"macro_"+sMacro+"(dbWIKI,f,'', '')"
					SetPara("ec 0")
				end
				if !bAllowed OR TDB_LastError then
					writeln(f, lang("badmacro"))
				end
			end
			write(f,"<!-- end of macro: "+sMacro +"//-->")
			close(f)
			WHILE RAMTEXT_Subst(sGebo, sOrigin, "ramtext:dynmacro") DO
			END
		else
			iEnd := iBeg +3
		end
	end
	CloseDb(dbWIKI)
endproc